//
//  TRMovieListingController.m
//  Tryst
//
//  Created by Francisco Tolmasky on Sun Feb 01 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import "NSArray+TRAdditions.h"
#import "TRMovie.h"

#import "TRMovieFileView.h"
#import "TRMovieInspector.h"
#import "TRMovieScrollView.h"
#import "TRMovieListingController.h"


@implementation TRMovieListingController

- (id)initWithCoder:(NSCoder *)aCoder
{
    self= [super initWithCoder: aCoder];
    if(self)
    {
        searchKey= [@"" retain];
        searchValue= [@"" retain];
        
        [self setPrimarySortKey: kTRMovieTitleProperty];
        
        isRearranging= NO;
    }
    return self;
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    [super observeValueForKeyPath: keyPath ofObject: object change: change context: context];
    if([keyPath isEqual: @"editable"] && !isRearranging) [self rearrangeObjects];
}

- (void)setSearchKey:(NSString *)aSearchKey
{
    searchKey= aSearchKey;
    [self rearrangeObjects];
}

- (void)setPrimarySortKey:(NSString *)aKey
{
    [self setSortDescriptors: [NSArray arrayWithObjects: 
        aKey==kTRMovieTitleProperty? 
            [[[NSSortDescriptor alloc] initWithKey: kTRMovieTitleProperty ascending: YES selector: @selector(caseInsensitiveCompare:)] autorelease]:
            [[[NSSortDescriptor alloc] initWithKey: kTRMovieGenreProperty ascending: YES selector: @selector(caseInsensitiveCompare:)] autorelease],
        [[[NSSortDescriptor alloc] initWithKey: kTRMovieYearProperty ascending: YES selector: @selector(caseInsensitiveCompare:)] autorelease],
        aKey==kTRMovieTitleProperty?
            [[[NSSortDescriptor alloc] initWithKey: kTRMovieGenreProperty ascending: YES selector: @selector(caseInsensitiveCompare:)] autorelease]:
            [[[NSSortDescriptor alloc] initWithKey: kTRMovieTitleProperty ascending: YES selector: @selector(caseInsensitiveCompare:)] autorelease], nil]];
    
    [self rearrangeObjects];
}

- (void)setSearchField:(NSSearchField *)aSearchField
{
    searchField= aSearchField;
}

- (NSSearchField *)searchField
{
    return searchField;
}

- (void)setMovieInspector:(TRMovieInspector *)aMovieInspector
{
    if (movieInspector)
        [movieInspector removeObserver: self forKeyPath: @"editable"];
    [movieInspector= aMovieInspector
        addObserver: self forKeyPath: @"editable"
            options: (NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld) context: NULL];
}

- (TRMovieInspector *)movieInspector
{
    return movieInspector;
}

- (IBAction)changeSearchKey:(id)sender
{
    [self setSearchKey: [sender representedObject]];
    
    [[[[searchField cell] searchMenuTemplate] itemAtIndex: [sender representedObject]==kTRMovieTitleProperty] setState: NSOffState];
    [[[[searchField cell] searchMenuTemplate] itemAtIndex: [sender representedObject]!=kTRMovieTitleProperty] setState: NSOnState];
    
    [[[sender menu] itemAtIndex: [sender representedObject]==kTRMovieTitleProperty] setState: NSOffState];
    [sender setState: NSOnState];
}

- (IBAction)search:(id)sender;
{
    [searchValue release];
    searchValue= [[sender stringValue] retain];
    [self rearrangeObjects];
}

- (void)add:(id)sender
{
    searchValue= @"";
    [searchField setStringValue: @""];
    [super add: sender];
	[self rearrangeObjects];
    if(![movieInspector isEditable]) [movieInspector setEditable: YES];
}

- (void)sheetDidEnd:(NSWindow *)sheet returnCode:(unsigned)returnCode contextInfo:(void *)contextInfo
{
	if(returnCode) [super remove: self];
}

- (void)remove:(id)sender
{
	NSBeginAlertSheet(
		@"Remove Movie",
		@"Yes", @"Cancel", nil,
		[movieInspector window],
		self,
		@selector(sheetDidEnd: returnCode: contextInfo:),
		NULL,
		NULL,
		NSLocalizedString(@"Are you sure you want to remove the selected items from the list?", @"Remove Warning"));
}

- (void)rearrangeObjects
{
	if(isRearranging) return;
	
    isRearranging= YES;
    [super rearrangeObjects];
    isRearranging= NO;
}

- (NSArray *)arrangeObjects:(NSArray *)objects
{
    objects= [super arrangeObjects: objects];
    
    if([searchKey isEqual: @""] || [searchValue isEqual: @""]) return objects;
    
    NSMutableArray *filteredObjects= [NSMutableArray arrayWithCapacity: [objects count]];
    NSEnumerator *objectEnumerator= [objects objectEnumerator];
    
    id object;

    while(object= [objectEnumerator nextObject])
	{
		NSString *searchString= [(TRMovie *)object valueForProperty: searchKey];
        if([searchString?searchString: @"" rangeOfString: searchValue options: NSCaseInsensitiveSearch].location != NSNotFound)
            [filteredObjects addObject: object];
	}
    return filteredObjects;
}

// Drag And Drop

/*- (BOOL)tableView:(NSTableView *)aTableView
	writeRows:(NSArray *)theRows
	toPasteboard:(NSPasteboard *)aPasteboard
{
	int i= 0,
		length= [theRows count];
	
	int row;
	NSMutableArray *movies= [[NSMutableArray alloc] initWithCapacity: length];
	NSArray *arrangedObjects= [self arrangedObjects];
	
	for(;i<length;++i) 
		if([[arrangedObjects objectAtIndex: row= [[theRows objectAtIndex: i] intValue]] filePath])
			[movies addObject: [NSString stringWithFormat: @"%i", row]];

	[aPasteboard declareTypes: [NSArray arrayWithObject: TRMoviesPboardType] owner: self];
	[aPasteboard setData: [NSKeyedArchiver archivedDataWithRootObject: movies] forType: TRMoviesPboardType];
	
	[movies release];
	
	return YES;
}*/

- (NSDragOperation)tableView:(NSTableView*)aTableView
	validateDrop:(id <NSDraggingInfo>)sender
	proposedRow:(int)aRow
	proposedDropOperation:(NSTableViewDropOperation)dropOperation
{
    NSArray *filenames;
	NSPasteboard *pasteboard= [sender draggingPasteboard];
	
	if( [sender draggingSource] == aTableView ||
		![[pasteboard types] containsObject: NSFilenamesPboardType] ||
		![filenames= [[pasteboard propertyListForType: NSFilenamesPboardType] pathsAndSubpathsMatchingExtensions: [TRMovie movieFileTypes]] count])
		return NSDragOperationNone;
	
	if([filenames count]>1 || dropOperation==NSTableViewDropAbove) [aTableView setDropRow: -1 dropOperation: NSTableViewDropOn];
	
    return NSDragOperationCopy;
}

- (BOOL)tableView:(NSTableView*)aTableView
	acceptDrop:(id <NSDraggingInfo>)sender
	row:(int)aRow
	dropOperation:(NSTableViewDropOperation)anOperation
{
	NSPasteboard *pasteboard= [sender draggingPasteboard];
	
	if(aRow==-1)
	{
		NSArray *filenames= [[pasteboard propertyListForType: NSFilenamesPboardType] pathsAndSubpathsMatchingExtensions: [TRMovie movieFileTypes]];
		NSMutableArray *movies= [[NSMutableArray alloc] initWithCapacity: [filenames count]];
		TRMutableMovie *movie;
		
		int i= 0,
			length= [filenames count];
		
		for(;i<length;++i)
		{
			movie= [[TRMutableMovie alloc] init];
			
			[movie setFilePath: [filenames objectAtIndex: i]];
			
			[movie setValue: [[[filenames objectAtIndex: i] lastPathComponent] stringByDeletingPathExtension] forProperty: kTRMovieTitleProperty];
			
			[movies addObject: [movie autorelease]];
		}
		
		[self addObjects: movies];
		
		[movies release];
	
		if(![movieInspector isEditable]) [movieInspector setEditable: YES];
	}
	else
	{
		[self setSelectionIndexes: [NSIndexSet indexSetWithIndex: aRow]];
		
		[[(TRMovieScrollView *)[movieInspector enclosingScrollView] fileView] updateModelWithFilePath:
			[[pasteboard propertyListForType: NSFilenamesPboardType] firstPathOrSubpathMatchingExtensions: [TRMovie movieFileTypes]]];
	}
	
	return YES;
}

@end