//
//  TRMultiValue.m
//  Tryst
//
//  Created by Francisco Tolmasky on Tue Dec 30 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "TRMultiValue.h"

@implementation TRMultiValue

- (id)init
{
    self= [super init];
    if(self)
    {
        identifiers= [[NSMutableArray allocWithZone: [self zone]] init];
        labels= [[NSMutableArray allocWithZone: [self zone]] init];
        values= [[NSMutableArray allocWithZone: [self zone]] init];
    }
    return self;
}

- (id)initWithMultiValue:(TRMultiValue *)aMultiValue
{
    self= [super init];
    if(self)
    {
        identifiers= [[aMultiValue valueForKey: @"identifiers"]/*->identifiers*/ copyWithZone: [self zone]];
        labels= [[aMultiValue valueForKey: @"labels"] /*->labels*/ copyWithZone: [self zone]];
        values= [[aMultiValue valueForKey: @"values"]/*->values*/ copyWithZone: [self zone]];
    }
    return self;
}

- (id)initWithCoder:(NSCoder *)aCoder
{
	self= [super init];
	if(self)
	{
		if ( [aCoder allowsKeyedCoding] )
		{
			labels= [[aCoder decodeObjectForKey: @"TRLabels"] retain];
			values= [[aCoder decodeObjectForKey: @"TRValues"] retain];
		}
		else
		{
			//[NSException raise: NSInvalidArchiveOperationException format: @"TRMovie only supports NSKeyedArchiver coders."];
			labels= [[aCoder decodeObject] retain];
			values= [[aCoder decodeObject] retain];
		}
		
		identifiers= [[NSMutableArray alloc] init];
		int i= 0,
			length= [labels count];
				
		for(;i<length;++i) [identifiers addObject: [NSString stringWithFormat: @"%i", i]];
	}
	return self;
}

- (void)dealloc
{
	[identifiers release];
    [labels release];
    [values release];
	
    [super dealloc];
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
    if ( [aCoder allowsKeyedCoding] )
	{
		[aCoder encodeObject: labels forKey: @"TRLabels"];
		[aCoder encodeObject: values forKey: @"TRValues"];
	}
	else
	{
		//[NSException raise: NSInvalidArchiveOperationException format: @"TRMovie only supports NSKeyedArchiver coders."];
		[aCoder encodeObject: labels];
		[aCoder encodeObject: values];
	}
	
	return;
}

- (id)replacementObjectForPortCoder:(NSPortCoder *)aCoder
{
	if ([aCoder isBycopy]) return self;
	return [super replacementObjectForPortCoder: aCoder];
} 

- (unsigned int)count
{
    return [values count];
}

- (id)valueAtIndex:(unsigned int)anIndex
{
    return [values objectAtIndex: anIndex];
}

- (NSString *)labelAtIndex:(unsigned int)anIndex
{
    return [labels objectAtIndex: anIndex];
}

- (NSString *)identifierAtIndex:(unsigned int)anIndex
{
    return [identifiers objectAtIndex: anIndex];
}
    
- (unsigned int)indexForIdentifier:(NSString *)anIdentifier
{
    return [identifiers indexOfObject: anIdentifier];
}

- (id)copyWithZone:(NSZone *)aZone
{
    return [[TRMultiValue allocWithZone: aZone] initWithMultiValue: self];
}

- (id)mutableCopyWithZone:(NSZone *)aZone
{
    return [[TRMutableMultiValue allocWithZone: aZone] initWithMultiValue: self];
}

@end

@implementation TRMutableMultiValue

- (NSString *)addValue:(id)aValue withLabel:(NSString *)aLabel
{
    NSString *identifier= [NSString stringWithFormat: @"%i", [identifiers count]];
    [identifiers addObject: identifier];
    [labels addObject: aLabel];
    [values addObject: aValue];
    return identifier;
}

- (NSString *)insertValue:(id)aValue withLabel:(NSString *)aLabel atIndex:(unsigned int)anIndex
{
    NSString *identifier= [NSString stringWithFormat: @"%i", [identifiers count]];
    [identifiers insertObject: identifier atIndex: anIndex];
    [labels insertObject: aLabel atIndex: anIndex];
    [values insertObject: aValue atIndex: anIndex];
    return identifier;
}

- (BOOL)removeValueAndLabelAtIndex:(unsigned int)anIndex
{
    if(anIndex<[identifiers count])
    {
        [identifiers removeObjectAtIndex: anIndex];
        [labels removeObjectAtIndex: anIndex];
        [values removeObjectAtIndex: anIndex];
        return YES;
    }
    return NO;
}

- (BOOL)replaceValueAtIndex:(unsigned int)anIndex withValue:(id)aValue
{
    BOOL exists= anIndex<[values count];
    [values replaceObjectAtIndex: anIndex withObject: aValue];
    return exists;
}

- (BOOL)replaceLabelAtIndex:(unsigned int)anIndex withLabel:(NSString*)aLabel
{
    BOOL exists= anIndex<[values count];
    [labels replaceObjectAtIndex: anIndex withObject: aLabel];
    return exists;
}

@end