//
//  TRBroadcasting.h
//  Tryst
//
//  Created by Francisco Tolmasky on Mon May 10 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>

extern NSString * const             kTRBroadcastServerProperty;

extern NSString * const				TRBroadcastServerInformationDidChangeNotification;
extern NSString * const				TRBroadcastClientInformationDidChangeNotification;

typedef enum
{
	TRBroadcastSuccessfulResponse=		 0,
	TRBroadcastPasswordInvalidResponse= -1,
	TRBroadcastConnectionLimitResponse= -2
}   TRBroadcastResponse;

@class  TRMutableMovie,
		TRBroadcastClient;

@protocol TRBroadcastNodeProtocol
- (bycopy)uniqueId;
@end

@interface TRBroadcastNode : NSObject <TRBroadcastNodeProtocol>
{
	NSString	*uniqueId;
}
- (NSString *)uniqueId;
@end

@protocol TRBroadcastServerProtocol
- (bycopy)movie;
@end

@interface TRBroadcastServer : TRBroadcastNode <TRBroadcastServerProtocol>
{	
	// Model
	struct
	{
		TRMutableMovie  *movie;
	
		NSTimeInterval  timeRemaining;
		unsigned		connectionLimit;
		NSString		*password;
	}					broadcast;
	
	// Networking
	NSTimer				*timer;
	
	NSSocketPort		*socket;
	NSConnection		*connection;
	
	NSNetService		*service;
	NSMutableDictionary *clients;
	NSMutableDictionary *subscriptions;
	
	NSString			*multicast;
}

- (TRMutableMovie *)movie;

- (NSTimeInterval)timeRemaining;

- (unsigned)connectionLimit;

- (unsigned)totalConnections;

- (void)broadcastMovie:(TRMutableMovie *)aMovie
	timeRemaining:(NSTimeInterval)aTimeInterval
	connectionLimit:(unsigned)aConnectionLimit
	password:(NSString *)aPassword;
	
- (BOOL)isPasswordProtected;

- (void)registerClient:(TRBroadcastClient *)aClient;
- (void)unregisterClient:(TRBroadcastClient *)aClient;
- (TRBroadcastResponse)subscribeClient:(TRBroadcastClient *)aClient withPassword:(NSString *)aPassword;

- (void)stop;

@end

@interface TRBroadcastClient : TRBroadcastNode
{
	NSMutableArray		*movies;
	NSNetServiceBrowser *browser;
}

- (NSArray *)movies;

- (void)close;
- (void)closeConnectionWithServer:(TRBroadcastServer *)aServer;

- (void) receiveStreamWithMulticastAddress:(NSString *)multicastAddress;

@end