//
//  TRMovieGutterView.m
//  Tryst
//
//  Created by Francisco Tolmasky on Mon Feb 02 2004.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//

#import "TRMultiValue.h"
#import "TRMovie.h"

#import "TRMovieInspector.h"
#import "TRMovieGutter.h"

@interface TRMovieInspector (friend)
- (NSString *)appendFieldFollowingFieldAtPoint:(NSPoint)aPoint;
- (void)clearFieldAtPoint:(NSPoint)aPoint;
- (NSString *)removeFieldAtPoint:(NSPoint)aPoint;

- (NSString *)currentEditedProperty;
- (NSString *)currentEditedIdentifier;
@end

@implementation TRMovieValueControl

- (id)initWithProperty:(NSString *)aProperty
    identifier:(NSString *)anIdentifier
    position:(TRMultiValuePosition)aPosition;
{
    self= [super initWithFrame: NSMakeRect(0.0,0.0,28.0,14.0)
                    mode: NSHighlightModeMatrix
                    cellClass: [NSButtonCell class]
                    numberOfRows: 1
                    numberOfColumns: 2];
    if(self)
    {
        property= aProperty;
        identifier= anIdentifier;
        
        nextValueControl= nil;
        
        [self setCellSize: NSMakeSize(14.0,14.0)];
        [self setIntercellSpacing: NSMakeSize(0.0,0.0)];
        
        NSButtonCell *lhsCell= [self cellAtRow: 0 column: 0];
        [lhsCell setTag: 0];
        [lhsCell setBordered: NO];
        [lhsCell setImage: [NSImage imageNamed: @"Remove Property"]];
        
        NSButtonCell *rhsCell= [self cellAtRow: 0 column: 1];
        [rhsCell setTag: 1];
        [rhsCell setTitle: @""];
        [rhsCell setEnabled: NO];
        [rhsCell setBordered: NO];
        
        [self setPosition: aPosition];
    }
    return self;
}

- (void)dealloc
{
	[nextValueControl release];
	
	[super dealloc];
}

- (void)removeFromSuperview
{
    if(nextValueControl) [nextValueControl removeFromSuperview];
    [super removeFromSuperview];
}

- (void)setProperty:(NSString *)aProperty
{
    property= aProperty;
}

- (NSString *)property
{
    return property;
}

- (void)setIdentifier:(NSString *)anIdentifier
{
    identifier= anIdentifier;
}

- (NSString *)identifier
{
    return identifier;
}

- (void)setPosition:(TRMultiValuePosition)aPosition
{
    position= aPosition;
}

- (TRMultiValuePosition)position
{
    return position;
}

- (TRMovieValueControl *)nextValueControl
{
    return nextValueControl;
}

- (void)setNextValueControl:(TRMovieValueControl *)aValueControl insert:(BOOL)insert
{
	if(nextValueControl==aValueControl) return;

    if(insert) [aValueControl setNextValueControl: nextValueControl insert: NO];
	[nextValueControl release];
	nextValueControl= [aValueControl retain];
}

@end

@implementation TRMovieGutter

- (id)initWithFrame:(NSRect)aFrame
{
    self= [super initWithFrame: aFrame];
    if(self)
    {
    }
    return self;
}

- (void)dealloc
{
	[valueControls.first release];
	
	[super dealloc];
}

- (BOOL)isFlipped
{
    return YES;
}

- (TRMovieValueControl *)valueControlForProperty:(NSString *)aProperty identifier:(NSString *)anIdentifier
{
    TRMovieValueControl *traverse= valueControls.first;
    while(traverse && !([[traverse property] isEqual: aProperty] && [[traverse identifier] isEqual: anIdentifier]))
        traverse= [traverse nextValueControl];
    return traverse;
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    if ([keyPath isEqual: @"currentEditedStartLocation"])
    {
        NSString *identifier= [movieInspector currentEditedIdentifier];
        if(identifier) valueControls.active= [self valueControlForProperty: [movieInspector currentEditedProperty] identifier: identifier];
        else valueControls.active= nil;
    }
    //else if ([keyPath isEqual: @"editable"] && [movieInspector isEditable]) [self loadValueControls];
    else if ([movieInspector isEditable]) [self loadValueControls];
}

- (void)setMovieInspector:(TRMovieInspector *)aMovieInspector
{
    if (movieInspector)
    {
        [movieInspector removeObserver: self forKeyPath: @"currentEditedStartLocation"];
        
        [movieInspector removeObserver: self forKeyPath: @"editable"];
        [movieInspector removeObserver: self forKeyPath: @"model"];
    }
    
    [movieInspector= aMovieInspector
        addObserver: self forKeyPath: @"currentEditedStartLocation"
            options: (NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld) context: NULL];
            
    [movieInspector
        addObserver: self forKeyPath: @"editable"
            options: (NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld) context: NULL];
    [movieInspector
        addObserver: self forKeyPath: @"model"
            options: (NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld) context: NULL];
}

- (TRMovieInspector *)movieInspector
{
    return movieInspector;
}

- (TRMovieValueControl *)loadValuesForProperty:(NSString *)aProperty
    followingValueControl:(TRMovieValueControl *)list
    correspondingRange:(NSRange *)range
    temporaryCount:(unsigned *)temporaryCount;
{
    NSTextContainer *textContainer= [movieInspector textContainer];
    NSLayoutManager *layoutManager= [movieInspector layoutManager];
    
    TRMultiValue *propertyList= [[movieInspector model] valueForProperty: aProperty];
    
    unsigned    i= 0,
                length= [propertyList count];
                
    TRMovieValueControl *last;
    
    for(;i<length || (i==length && i==0);++i,range->location= NSMaxRange(*range)+5)
    {
        last= list;
        list= list?[list nextValueControl]:valueControls.first;
        
        TRMultiValuePosition position=
            ((!last || [last position] & TRFinalValuePosition) ? TRInitialValuePosition:TRIntermediateValuePosition) |
            ((i==length-1 || length==0)?TRFinalValuePosition:TRIntermediateValuePosition);
        
        TRMovieValueControl *valueControl;
        
        NSString *identifier= [propertyList identifierAtIndex: i];
        if(!identifier) identifier= [[NSString stringWithFormat: @"t_%i", (*temporaryCount)++] retain];
        
        if(!list) 
        {
            valueControl= 
                [[TRMovieValueControl alloc] initWithProperty: aProperty identifier: identifier position: position];
            
            [valueControl setTarget: self];
            [valueControl setAction: @selector(concretizeControlMessage:)];
        }
        else
        {
            valueControl= list;
            [valueControl setProperty: aProperty];
            [valueControl setIdentifier: identifier];
            [valueControl setPosition: position];
        }

        [[movieInspector textStorage] attribute: kPropertyAttributeName atIndex: range->location effectiveRange: range];
        
        [valueControl setFrame:
            NSMakeRect( 0.0,last?NSMaxY([last frame])+(position&TRInitialValuePosition?14.0:0.0):[layoutManager boundingRectForGlyphRange: [layoutManager glyphRangeForCharacterRange: NSMakeRange(0,range->location-3) actualCharacterRange: NULL] inTextContainer: textContainer].size.height,
                        28.0,[layoutManager boundingRectForGlyphRange: [layoutManager glyphRangeForCharacterRange: *range actualCharacterRange: NULL] inTextContainer: textContainer].size.height)];
    
        NSButtonCell *valueCell= [valueControl cellAtRow: 0 column: 1];
        [valueCell setEnabled: i==length-1];
        [valueCell setImage: i==length-1?[NSImage imageNamed: @"Add Property"]:nil];

        BOOL isTemporaryIdentifier= [[valueControl identifier] characterAtIndex: 0]=='t';
        [valueCell= [valueControl cellAtRow: 0 column: 0] setEnabled: !isTemporaryIdentifier];
        [valueCell setImage: isTemporaryIdentifier?nil:[NSImage imageNamed: @"Remove Property"]];

        if(!last) valueControls.first= valueControl;
        
        if(!list)
        {
            [last setNextValueControl: valueControl insert: NO];
            [self addSubview: valueControl];
            
            [valueControl release];
        }
        
        list= valueControl;
    }
    
    return list;
}

- (void)loadValueControls
{
    TRMovie *model= [movieInspector model];
	
    NSString	*title= [model valueForProperty: kTRMovieTitleProperty],
				*year= [model valueForProperty: kTRMovieYearProperty],
				*genre= [model valueForProperty: kTRMovieGenreProperty];
	
	
    TRMovieValueControl *current;
    
    NSRange range=  NSMakeRange((title?[title length]:11)+(year?[year length]:4)+(genre?[genre length]:5)+18,0);
    
    unsigned temporaryCount= 0;
    
    current= [self loadValuesForProperty: kTRMovieCrewProperty followingValueControl: nil correspondingRange: &range temporaryCount: &temporaryCount];
    valueControls.last= [self loadValuesForProperty: kTRMovieCastProperty followingValueControl: current correspondingRange: &range temporaryCount: &temporaryCount];

    if([valueControls.last nextValueControl])
    {
        [[valueControls.last nextValueControl] removeFromSuperview];
        [valueControls.last setNextValueControl: nil insert: NO];
    }
    
    [self setNeedsDisplay: YES];
}

- (void)shiftValueControlsAfterValueControl:(TRMovieValueControl *)list by:(float)shift
{
    TRMovieValueControl *shiftValueControl= list?list:valueControls.first;
        
    while(shiftValueControl= [shiftValueControl nextValueControl])
        [shiftValueControl setFrameOrigin: NSMakePoint(0.0,[shiftValueControl frame].origin.y+shift)];

    if(list)
        //[self setNeedsDisplayInRect: NSMakeRect(0.0,[list?list:valueControls.first frame].origin.y,28.0,NSMaxY([valueControls.last frame])-[list?list:valueControls.first frame].origin.y+(shift<0?-shift:0.0))];
        [self setNeedsDisplayInRect: NSMakeRect(0.0,[list frame].origin.y,28.0,NSMaxY([valueControls.last frame])-[list frame].origin.y+(shift<0?-shift:0.0))];
    else
        [self setNeedsDisplay: YES];
}

- (void)updateValueControlForProperty:(NSString *)aProperty identifier:(NSString *)anIdentifier withValue:(NSString *)aStringValue height:(float)aHeight
{
    BOOL proposedRemovableState= aStringValue?1:0;

    TRMovieValueControl *valueControl= [self valueControlForProperty: aProperty identifier: anIdentifier];
    NSButtonCell    *addValueCell= [valueControls.active cellAtRow: 0 column: 1],
                    *removeValueCell= [valueControls.active cellAtRow: 0 column: 0];

    // Update Height of Value Control
    // thought never but did (empty this was my friend) (not gone though)yeah
    float shift= aHeight-(valueControl?[valueControl frame].size.height:[valueControls.first frame].origin.y);

    if(shift)
    {
        if(valueControl) [valueControl setFrameSize: NSMakeSize(28.0,aHeight)];
        else [valueControls.first setFrameOrigin: NSMakePoint(0.0,aHeight)];
        
        [self shiftValueControlsAfterValueControl: valueControl by: shift];
    }

    // Update State of Value Control
    
    if ([addValueCell isEnabled]!=proposedRemovableState &&
        [valueControl position] & TRFinalValuePosition)
    {
        [addValueCell setImage: proposedRemovableState?[NSImage imageNamed: @"Add Property"]:nil];
        [addValueCell setEnabled: proposedRemovableState];
    }
    
    if ([removeValueCell isEnabled]!=proposedRemovableState && 
        [valueControl position]==TRIntermediateValuePosition || [valueControl position] & TRInitialValuePosition)
    {
        [removeValueCell setImage: proposedRemovableState?[NSImage imageNamed: @"Remove Property"]:nil];
        [removeValueCell setEnabled: proposedRemovableState];
    }
}

- (void)updateActiveValueControlWithValue:(NSString *)aStringValue height:(float)aHeight
{
    BOOL proposedRemovableState= aStringValue?1:0;

    NSButtonCell    *addValueCell= [valueControls.active cellAtRow: 0 column: 1],
                    *removeValueCell= [valueControls.active cellAtRow: 0 column: 0];

    // Update Height of Value Control
    
    float shift= aHeight-(valueControls.active?[valueControls.active frame].size.height:[valueControls.first frame].origin.y);

    if(shift)
    {
        if(valueControls.active) [valueControls.active setFrameSize: NSMakeSize(28.0,aHeight)];
        else [valueControls.first setFrameOrigin: NSMakePoint(0.0,aHeight)];
        
        [self shiftValueControlsAfterValueControl: valueControls.active by: shift];
       
    }
    
    if(!valueControls.active) return;

    // Update State of Value Control
    
    if ([addValueCell isEnabled]!=proposedRemovableState &&
        [valueControls.active position] & TRFinalValuePosition)
    {
        [addValueCell setImage: proposedRemovableState?[NSImage imageNamed: @"Add Property"]:nil];
        [addValueCell setEnabled: proposedRemovableState];
    }
    
    if ([removeValueCell isEnabled]!=proposedRemovableState && 
        [valueControls.active position]==TRIntermediateValuePosition || [valueControls.active position] & TRInitialValuePosition)
    {
        [removeValueCell setImage: proposedRemovableState?[NSImage imageNamed: @"Remove Property"]:nil];
        [removeValueCell setEnabled: proposedRemovableState];
    }
}

- (void)concretizeControlMessage:(id)sender
{
    NSButtonCell *actionCell= [sender selectedCell];
    
    if([actionCell tag]) //add property
    {
        NSString *identifier= [movieInspector appendFieldFollowingFieldAtPoint: NSMakePoint(kLeftTabLocation,[sender frame].origin.y+7.0)];
    
        TRMovieValueControl *valueControl= [[TRMovieValueControl alloc] initWithProperty: [sender property] identifier: identifier position: TRFinalValuePosition];        
        [valueControl setTarget: self];
        [valueControl setAction: @selector(concretizeControlMessage:)];
        
        // Adjust previous final control
        NSButtonCell * previousAddButton= [sender cellAtRow: 0 column: 1];
        
        [sender setNextValueControl: valueControl insert: YES];
        [sender setPosition: [sender position]&~TRFinalValuePosition];
        
        [previousAddButton setImage: nil];
        [previousAddButton setEnabled: NO];
        
        //Position relative to others
        [valueControl setFrameOrigin: NSMakePoint(0.0,NSMaxY([sender frame]))];
        
        [self shiftValueControlsAfterValueControl: valueControl by: 14.0];

        valueControls.active= valueControl;
        if(![valueControl nextValueControl]) valueControls.last= valueControl;
        
        [self addSubview: valueControl];
        
        [valueControl release];
    }
    else
    {
        if([sender position]&TRFinalValuePosition && !([sender position]&TRInitialValuePosition))
        {
            TRMovieValueControl *previousValueControl= valueControls.first;
            
            while([previousValueControl nextValueControl]!=sender) previousValueControl= [previousValueControl nextValueControl];
            
            // Set up position first, state information will be handled by selecting it.
            [previousValueControl setPosition: [previousValueControl position]|TRFinalValuePosition];
            [previousValueControl setNextValueControl: [sender nextValueControl] insert: NO];
			
			NSString *previousValue= [movieInspector removeFieldAtPoint: NSMakePoint(kLeftTabLocation,[sender frame].origin.y+7.0)]; //To avoid being skipped during overlaoding
			
			if( !([previousValueControl position]&TRInitialValuePosition) || previousValue)
			{
				[[previousValueControl cellAtRow: 0 column: 0] setEnabled: YES];
				[[previousValueControl cellAtRow: 0 column: 0] setImage: [NSImage imageNamed: @"Remove Property"]];
			}
			
			if(sender==valueControls.last) valueControls.last= previousValueControl;
            
            [self shiftValueControlsAfterValueControl: sender by: -[sender frame].size.height];
            
            [sender setNextValueControl: nil insert: NO];
            [sender removeFromSuperview];
        }
        else [movieInspector clearFieldAtPoint: NSMakePoint(kLeftTabLocation,[sender frame].origin.y+7.0)];
    }
}

@end