//
//  TRRolloverButton.m
//  Tryst
//
//  Created by Francisco Tolmasky on Sun Mar 14 2004.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//

#import "TRRolloverButton.h"

@interface TRRolloverButtonCell (Private)
- (void)_setRollover:(BOOL)flag;
@end

@implementation TRRolloverButtonCell

- (void)setIlluminated:(BOOL)flag
{
	flags.illuminated= flag;
}

- (void)_setRollover:(BOOL)flag
{
	flags.rollover= flag;
}

- (void)setImage:(NSImage *)anImage
{
	if([self image]==anImage) return;
	
	NSImage *alternateImage= [[NSImage alloc] initWithSize: NSMakeSize(18.0,18.0)];
	
	[alternateImage lockFocus];
	[[NSColor whiteColor] set];
	NSRectFill(NSMakeRect(0.0,0.0,18.0,18.0));
	[anImage compositeToPoint: NSMakePoint(0.0,0.0) operation: NSCompositeDestinationIn];
	[alternateImage unlockFocus];

	[super setImage: anImage];
	[super setAlternateImage: alternateImage];
}

- (void)drawWithFrame:(NSRect)aFrame inView:(NSView *)controlView
{
	[NSGraphicsContext saveGraphicsState];
	if(flags.rollover || [self isHighlighted])
	{
		NSBezierPath *path= [NSBezierPath bezierPathWithOvalInRect: aFrame];
		
		[[[NSColor blackColor] colorWithAlphaComponent: [self isHighlighted]?0.6f:0.5f] set];
		
		[path fill];
	}
	
	[self drawInteriorWithFrame: aFrame inView: controlView];
	
	[NSGraphicsContext restoreGraphicsState];
}

- (void)drawInteriorWithFrame:(NSRect)aFrame inView:(NSView *)controlView
{	
	[(flags.illuminated || flags.rollover || [self isHighlighted])?[self alternateImage]:[self image]
		compositeToPoint: NSMakePoint(aFrame.origin.x,aFrame.origin.y+[self cellSize].height)
		operation: NSCompositeSourceOver];
}

- (BOOL)trackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(NSView *)controlView untilMouseUp:(BOOL)untilMouseUp
{
	BOOL	done= NO,
			result= NO;
	NSPoint currentPoint = [theEvent locationInWindow];
	
	BOOL trackContinously= [self startTrackingAt: currentPoint inView: controlView];
	// catch next mouse-dragged or mouse-up event until timeout
	BOOL mouseIsUp= NO;
	NSEvent *nextEvent;
	while (!done)
	{
		NSPoint lastPoint= currentPoint;
		nextEvent= [NSApp nextEventMatchingMask: (NSLeftMouseUpMask|NSLeftMouseDraggedMask) untilDate: [NSDate distantFuture] inMode: NSEventTrackingRunLoopMode dequeue: YES];
		if (nextEvent)
		{
			currentPoint= [nextEvent locationInWindow];
			if (trackContinously)
			{
				if (![self continueTracking: lastPoint at: currentPoint inView: controlView])
				{
					done= YES;
					[self stopTracking: lastPoint at: currentPoint inView: controlView mouseIsUp: mouseIsUp];
				}
				if ([self isContinuous]) [NSApp sendAction: [self action] to: [self target] from: self];
			}
			
			mouseIsUp= ([nextEvent type] == NSLeftMouseUp);
			
			done= done || mouseIsUp;
			
			if (untilMouseUp) result= mouseIsUp;
			else
			{
				// check, if the mouse left our cell rect
				result= NSPointInRect([controlView convertPoint: currentPoint fromView: nil], cellFrame);
				if (!result) 
				{
					done= YES;
					flags.rollover= NO;
				}
				else flags.rollover= YES;
			}
			if (done && result && ![self isContinuous]) [NSApp sendAction: [self action] to: [self target] from: self];
		}
		else return YES;
	}
	return result;
}

@end

@implementation TRRolloverButton

+ (void)initialize
{
	if(self==[TRRolloverButton class]) [self setCellClass: [TRRolloverButtonCell class]];
}

- (id)initWithFrame:(NSRect)aFrame
{
	self= [super initWithFrame: aFrame];
	if(self)
	{
		[self setBordered: NO];
		[self setButtonType: NSMomentaryChangeButton];
		[self setTitle: @""];
	}
	return self;
}

- (void)setIlluminated:(BOOL)flag
{
	[[self cell] setIlluminated: flag];
	
	[self setNeedsDisplay: YES];
}

- (void)resetCursorRects
{
    [super resetCursorRects];
    
	[super removeTrackingRect: trackingRectTag];
          
    trackingRectTag= [self addTrackingRect: [self visibleRect] owner: self userData: nil assumeInside: NO];
}

- (void)mouseEntered:(NSEvent *)anEvent
{
	[[self cell] _setRollover: YES];
	
	[self setNeedsDisplay: YES];
}

- (void)mouseExited:(NSEvent *)anEvent
{
	[[self cell] _setRollover: NO];

	[self setNeedsDisplay: YES];
}

@end
