//
//  TRWindowController.h
//  Tryst
//
//  Created by Francisco Tolmasky on Tue Nov 18 2003.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//

#import <AppKit/AppKit.h>


@class  TRController,
        TRMovieTableView,
        TRMovieInspector,
		TRMovieScrollView,
		TRIntegerControl,
		TRTimeIntervalControl,
        TRMovieListingController,
		TRBroadcastManager;
		
typedef enum
{
	TRWindowLibraryDisplayMode= 0,
	TRWindowBrowseDisplayMode=  1
} TRWindowDisplayMode;

@interface TRWindowController : NSWindowController
{
	TRWindowDisplayMode					displayMode;
	
    BOOL								isCollapsed;
    BOOL								collapseInProgress;
	
    float								movieListRowHeight;
    
	NSTimer								*movieListingTimer;
	
    //Model
	TRController						*controller;
    
    //Views
	IBOutlet NSTabView					*displayModeTabView;
	
	IBOutlet NSSearchField				*searchField;
	
    IBOutlet NSSplitView				*mainSplitView;
    IBOutlet NSView						*searchContainer;
	IBOutlet NSView						*controlsContainer;
	
	//IBOutlet NSView					*toolbarContainer;
	IBOutlet NSButton					*broadcastButton;
	IBOutlet NSButton					*subscribeButton;
	
    IBOutlet NSButton					*editButton;
    
    IBOutlet TRMovieTableView			*movieListing;
    IBOutlet TRMovieInspector			*movieInspector;
    IBOutlet TRMovieScrollView			*movieScrollView;
	
	IBOutlet NSSecureTextField			*passwordEntryTextField;
	IBOutlet NSPanel					*broadcastSheet;
	IBOutlet NSPanel					*passwordSheet;
    
	IBOutlet TRTimeIntervalControl		*timeRemainingControl;
	IBOutlet NSButton					*connectionLimitCheckBox;
	IBOutlet NSButton					*requirePasswordCheckBox;
	IBOutlet TRIntegerControl			*connectionLimitControl;
	IBOutlet NSSecureTextField			*passwordTextField;
	
    //Controllers
	IBOutlet TRMovieListingController   *movieListingController;
    IBOutlet NSObjectController			*inspectorController;
}

- (id)initWithController:(TRController *)aController;

- (void)setCollapsed:(BOOL)collapseFlag;
- (BOOL)collapsed;

- (void)setDisplayMode:(TRWindowDisplayMode)aDisplayMode;
- (TRWindowDisplayMode)displayMode;

// Movie Inspector

- (IBAction)toggleMovieInspector:(id)sender;

// Broadcast

- (IBAction)synchronizeBroadcastOptions:(id)sender;
- (IBAction)runBroadcastSheet:(id)sender;
- (IBAction)endBroadcastSheet:(id)sender;

- (IBAction)handleBroadcast:(id)sender;

- (IBAction)endPasswordSheet:(id)sender;

// Listening

- (IBAction)subscribe:(id)sender;

// Bug Reporting

- (IBAction)reportBug:(id)sender;

@end
